-- 1. Create Table
--NOTE: Must change "DBUSER" to valid oracle user name

CREATE TABLE “DBUSER”.”BULK_EXAMPLE" 
   (	"OBJECTID_1" NUMBER(10,0), 
	"OBJECTID_2" NUMBER(9,0), 
	"OBJECTID" NUMBER(9,0), 
	"PRE_DIR" CHAR(2 BYTE), 
	"STR_NAME" CHAR(20 BYTE), 
	"STR_TYPE" CHAR(4 BYTE), 
	"SUF_DIR" CHAR(2 BYTE), 
	"ROUTE_NUM" NUMBER(4,0), 
	"USE_RATING" CHAR(2 BYTE), 
	"SHAPE_LENG" NUMBER(31,11), 
	"PRE_TYPE" CHAR(15 BYTE), 
	"CENSUS_ID" NUMBER(9,0), 
	"SHAPE_LE_1" NUMBER(31,11), 
	"SHAPE_LEN" NUMBER(31,11), 
	"GEOM" "MDSYS"."SDO_GEOMETRY" 
   )
 

   
-- 2. Create table for IDs
CREATE TABLE BULK_EXAMPLE_TEMP ("OBJECTID" NUMBER(9,0), "OPERATION" VARCHAR2(25 BYTE));

-- 3. Turn off DEFINITIONS in SQL Statements (this allows to send ampersand in the HTTP request)
SET DEF OFF;

-- 4. Create Trigger INSERT on BULK_EXAMPLE table
-- This trigger inserts the id of every new feature in the temporary table
CREATE OR REPLACE TRIGGER TRG_BULK_EXAMPLE_INSERT 
AFTER INSERT ON BULK_EXAMPLE
FOR EACH ROW 
BEGIN
     INSERT INTO BULK_EXAMPLE_TEMP VALUES (:new.objectid_1, 'INSERT');
END TRG_BULK_EXAMPLE_INSERT;


-- 5. Create Trigger UPDATE on BULK_EXAMPLE table
-- This trigger inserts the id of every updated feature in the temporary table 
CREATE OR REPLACE TRIGGER TRG_BULK_EXAMPLE_UPDATE
AFTER UPDATE ON BULK_EXAMPLE
FOR EACH ROW 
BEGIN
     INSERT INTO BULK_EXAMPLE_TEMP VALUES (:new.objectid_1, 'UPDATE');
END TRG_BULK_EXAMPLE_UPDATE;

-- 6. Create Trigger DELETE on BULK_EXAMPLE table
-- This trigger inserts the id of every updated feature in the temporary table 
CREATE OR REPLACE TRIGGER TRG_BULK_EXAMPLE_DELETE
AFTER DELETE ON BULK_EXAMPLE
FOR EACH ROW 
BEGIN
     INSERT INTO BULK_EXAMPLE_TEMP VALUES (:old.objectid_1, 'DELETE');
END TRG_BULK_EXAMPLE_DELETE;

-- 7. Create HTTP function
-- This function reads the temporary ids out of the database and sends it to a FME Server workspace
-- The FME Server URL has to be adapted.
CREATE OR REPLACE FUNCTION FUNC_BULK_EXAMPLE_HTTP RETURN char
IS 
  -- Define type to store temporary IDs
  TYPE idSet IS TABLE OF BULK_EXAMPLE_TEMP%ROWTYPE;
  writtenIds idSet;
  
  -- CLOB is necessary because of the length limit of VARCHAR
  list_val CLOB;
  message CLOB;
  operation VARCHAR2(25 byte);
  
  -- UTL_HTTP vars
  req utl_http.req;
  resp utl_http.resp;
  
BEGIN
	-- Read temporary stored IDs
    SELECT objectid, operation BULK COLLECT INTO writtenids FROM BULK_EXAMPLE_TEMP;
    
	-- Write the values in a CLOB (string)
    FOR i IN writtenids.FIRST .. writtenids.LAST
    LOOP
      IF i != writtenids.LAST THEN
        list_val := list_val || TO_CLOB('"' || writtenids(i).objectid || '",');
      ELSE
        list_val := list_val || TO_CLOB('"' || writtenids(i).objectid || '"');
        operation := writtenids(i).operation;
      END IF;
      
    END LOOP ;
    
	-- Generate a message string
     message := 'operation='||operation||'&idlist='||list_val;
    
    -- Call the Logger workspace
    -- Because of the length of the message it is necessary to send a POST request
    req := utl_http.begin_request('http://SWISSCHEESE/fmejobsubmitter/trigger/oracle_bulk_logger.fmw','POST');
    utl_http.set_header(req, 'User-Agent', 'Mozilla/4.0');
    utl_http.set_header(req,'Content-Type','application/x-www-form-urlencoded');
    utl_http.set_header(req, 'content-length', length(message));
    utl_http.write_text(req, message);
    utl_http.end_request(req);

    
    -- Dummy return value
    RETURN 'Y';
EXCEPTION
    WHEN utl_http.end_of_body THEN
		utl_http.end_response(resp);
    WHEN OTHERS THEN  
    -- Dummy return value
    RETURN 'F';
END FUNC_BULK_EXAMPLE_HTTP;
